﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using DbExample.Web.Models;
using DbExample.Web.Services;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.FormatProviders;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.FormatProviders.OpenXml.Docx;
using Telerik.Windows.Documents.UI;

namespace DbExample
{
    public partial class MainPage : UserControl
    {
        #region Constructors

        public MainPage()
        {
            StyleManager.ApplicationTheme = new Windows7Theme();
            InitializeComponent();
        }

        #endregion

        #region Events

        private void documentDomainDataSource_LoadedData(object sender, LoadedDataEventArgs e)
        {
            if (e.HasError)
            {
                System.Windows.MessageBox.Show(e.Error.ToString(), "Load Error", System.Windows.MessageBoxButton.OK);
                e.MarkErrorAsHandled();
            }
        }

        private void listControl_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 1)
            {
                Document doc = e.AddedItems[0] as Document;
                if (doc != null)
                {
                    this.editor.Document = GetDocumentFromDocx(doc.Content);
                    this.ShowEditor();
                }
                else
                {
                    this.HideEditor();
                }
            }
        }

        private void btnNew_Click(object sender, RoutedEventArgs e)
        {
            NewDocumentWindow dialog = new NewDocumentWindow();
            dialog.Closed += new EventHandler(newDoc_Closed);
            dialog.Show();
        }

        void newDoc_Closed(object sender, EventArgs e)
        {
            NewDocumentWindow dialog = (NewDocumentWindow)sender;
            string title = dialog.Title;
            if (!string.IsNullOrEmpty(title))
            {
                Document newDoc = new Document();
                newDoc.Title = title;
                newDoc.Content = GetDocxFromDocument(new RadDocument());

                DocumentsDomainContext context = (DocumentsDomainContext)documentDomainDataSource.DomainContext;
                context.Documents.Add(newDoc);
                documentDomainDataSource.SubmitChanges();

                this.Dispatcher.BeginInvoke(() =>
                {
                    this.documentListBox.SelectedItem = newDoc;
                });
            }

        }

        private void Save_Click(object sender, RoutedEventArgs e)
        {
            if (documentDomainDataSource.IsBusy)
            {
                return;
            }
            Document doc = documentListBox.SelectedItem as Document;
            if (doc != null)
            {
                doc.Content = GetDocxFromDocument(this.editor.Document);
                documentDomainDataSource.SubmitChanges();
            }
        }

        #endregion

        #region Methods

        private void HideEditor()
        {
            this.editor.Visibility = Visibility.Collapsed;
            this.radRichTextBoxRibbonUI.Visibility = Visibility.Collapsed;
        }

        private void ShowEditor()
        {
            this.editor.Visibility = Visibility.Visible;
            this.radRichTextBoxRibbonUI.Visibility = Visibility.Visible;
        }

        private static byte[] GetDocxFromDocument(RadDocument document)
        {
            IDocumentFormatProvider exporter = new DocxFormatProvider();
            MeasureAndArrangeInDefaultSize(document);
            byte[] result = exporter.Export(document);
            return result;
        }

        private static RadDocument GetDocumentFromDocx(byte[] docx)
        {
            RadDocument result;
            if (docx == null || docx.Length == 0)
            {
                result = new RadDocument();
            }
            else
            {
                IDocumentFormatProvider provider = new DocxFormatProvider();
                result = provider.Import(docx);
            }
            return result;
        }

        private static void MeasureAndArrangeInDefaultSize(RadDocument document)
        {
            document.Arrange(new RectangleF(PointF.Empty, RadDocument.MAX_DOCUMENT_SIZE));
            document.Measure(RadDocument.MAX_DOCUMENT_SIZE);
        }

        #endregion

        #region RibbonUIEvents

        private bool IsSupportedImageFormat(string extension)
        {
            if (extension != null)
            {
                extension = extension.ToLower();
            }

            return extension == ".jpg" ||
                extension == ".jpeg" ||
                extension == ".png" ||
                extension == ".bmp";
        }

        private void Statistics_Click(object sender, RoutedEventArgs e)
        {
            string statistics = this.editor.Document.GetStatisticsInfo().ToString();
            MessageBox.Show(statistics, "Document Statistics", MessageBoxButton.OK);
        }

        private void ViewDocumentStructure_Click(object sender, RoutedEventArgs e)
        {
            RadWindow window = new RadWindow();

            DocumentModelTreeViewer documentTreeViewer = new DocumentModelTreeViewer();
            documentTreeViewer.DisplayDocumentLayoutTree(this.editor.Document);

            window.Content = documentTreeViewer;

            window.WindowStartupLocation = Telerik.Windows.Controls.WindowStartupLocation.CenterOwner;
            window.Width = 600;
            window.Height = 750;

            window.ShowDialog();
        }

        private void editor_Drop(object sender, DragEventArgs e)
        {
            FileInfo[] droppedFiles = e.Data.GetData(DataFormats.FileDrop) as FileInfo[];

            foreach (FileInfo droppedFile in droppedFiles)
            {
                if (this.IsSupportedImageFormat(droppedFile.Extension))
                {
                    using (Stream imageStream = droppedFile.OpenRead())
                    {
                        this.editor.InsertImage(imageStream, droppedFile.Extension);
                    }
                }
            }
        }
        #endregion
    }
}
